var Middlewares = function(object) {}

var gFunctionAuthenticator = require('letsnet-graphed').functionAuth

var mongoose = require('mongoose')
var AccountSchema = new mongoose.Schema({
    id: mongoose.Schema.Types.ObjectId,
    name: String,
    title: String,
    apikeys: [{
        key: String,
        email: String
    }],
    created: Date,
    url: {
        test: String,
        release: String,
        production: String
    },
    mongooseCredentials: {
        url: String,
        user: String,
        passwd: String
    },
    digitalOcean: {
        dropletId: String
    },
    mail: {
        mail: String,
        password: String,
        clientID: String,
        clientSecret: String,
        refreshToken: String,
        from: String
    }
})

Middlewares.prototype.gAuthenticateFunction = function(object, queue) {
    gFunctionAuthenticator({appid: object.appid || '---'}).authenticateFunction({functionID: object.functionID || '---', email: object.email || '---'}, function(error, document) {
        delete object['functionID']; delete object['appid']
        if(error) queue.stop({lCode: document.lCode, msg: document.msg, type: 'authentication-error'})
        else queue.next()
    })
}

Middlewares.prototype.getMongooseCredentials = function(object, queue) {
    if(object.mongooseCredentials === undefined) queue.stop({type: 'credential-error', msg: 'Adja meg a kapcsolathoz szükséges információkat!'})
    else {
        let mongodb = mongoose.createConnection('mongodb://' + object.mongooseCredentials.user + ':' + object.mongooseCredentials.passwd + object.mongooseCredentials.url)
        delete object['mongooseCredentials']
        let shema = mongodb.model('account', AccountSchema)
        shema.findOne({apikeys: {$elemMatch: {$or: [{email: object.email }, {key: object.apikey}]}}}, 'mongooseCredentials', function(err, document) {
            if(err) queue.stop({type: 'connection-error', msg: 'Hiba az adatbázis kapcsolatnál!'})
            else {   
                mongoose.disconnect()             
                if(document == null) { queue.stop({type: 'query-error', msg: 'Nem található az adatbázisban!'})}
                else if(document.mongooseCredentials.url === undefined || document.mongooseCredentials.user === undefined || document.mongooseCredentials.passwd === undefined) queue.stop({type: 'query-error', msg: 'Nem találhatók az adatbázis kapcsolathoz szükséges információk!'})
                else {
                    object.userMongooseCredentials = document.mongooseCredentials
                    queue.next()
                }
            }
        })
    }
}

Middlewares.prototype.getMongooseCredentialsByAppID = function(object, queue) {
    if(object.mongooseCredentials === undefined) queue.stop({type: 'credential-error', msg: 'Adja meg a kapcsolathoz szükséges információkat!'})
    else {
        let mongodb = mongoose.createConnection('mongodb://' + object.mongooseCredentials.user + ':' + object.mongooseCredentials.passwd + object.mongooseCredentials.url)
        delete object['mongooseCredentials']
        let shema = mongodb.model('account', AccountSchema)
        shema.findOne({apikeys: {$elemMatch: {key: object.apikey}}}, 'mongooseCredentials', function(err, document) {
            if(err) queue.stop({type: 'connection-error', msg: 'Hiba az adatbázis kapcsolatnál!'})
            else {   
                mongoose.disconnect()             
                if(document == null) { queue.stop({type: 'query-error', msg: 'Nem található az adatbázisban!'})}
                else if(document.mongooseCredentials.url === undefined || document.mongooseCredentials.user === undefined || document.mongooseCredentials.passwd === undefined) queue.stop({type: 'query-error', msg: 'Nem találhatók az adatbázis kapcsolathoz szükséges információk!'})
                else {
                    object.userMongooseCredentials = document.mongooseCredentials
                    queue.next()
                }
            }
        })
    }
}

Middlewares.prototype.prepareApikey = function(object, queue) {
    if(object.apikey === undefined) queue.stop({type: 'authentication-error', msg: 'Nem található apikey!'})
    else {
        object.apikey = object.apikey.slice(7)
        queue.next()
    }
}

module.exports = Middlewares